package jp.co.powerbeans.vbscore.db;

import java.util.HashMap;
import java.util.Map;

/**
 * DBManager の Cache 対応版
 * @author amonden
 */
public final class DBCacheManager {
	/**
	 * キャッシュ用Map. key = メソッド名 + cls + 検索パラメータ
	 */
	private static Map<String, Object> dbcache = new HashMap<String, Object>();

	/**
	 * 条件とソート順を指定して複数件取得
	 * @param cls モデルクラス
	 * @param where 検索条件
	 * @param orderby ソート順
	 * @return 検索結果
	 */
	public static Object[] findBy(Class cls, String where, String orderby) {
		String key = "findBy" + cls + where + orderby;
		Object[] res = (Object[]) getCache(key);
		if (res == null) {
			res = DBManager.findBy(cls, where, orderby);
			registCache(key, res); // TODO clone した方が良いかも
		}
		return res;
	}

	/**
	 * key と 結果を渡してキャッシュする
	 * @param key キー
	 * @param res レスポンス
	 */
	private static void registCache(String key, Object res) {
		dbcache.put(key, res);
	}

	/**
	 * key からキャッシュする
	 * @param key キー
	 * @return レスポンス
	 */
	private static Object getCache(String key) {
		return dbcache.get(key);
	}

	/**
	 * findByPrimaryKey の結果をキャッシュし返す
	 * @param cls Bean
	 * @param id 検索条件
	 * @return 検索結果
	 */
	public static Object findByPrimaryKey(Class cls, int id) {
		String key = "findByPrimaryKey" + cls + id;
		Object res = getCache(key);
		if (res == null) {
			res = DBManager.findByPrimaryKey(cls, id);
			registCache(key, res); // TODO clone した方が良いかも
		}
		return res;
	}

	/**
	 * キャッシュを全てクリア
	 */
	public static void clear() {
		dbcache.clear();
	}

	/**
	 * findByAll の結果をキャッシュし返す
	 * @param cls Bean
	 * @return 検索結果
	 */
	public static Object[] findByAll(Class cls) {
		String key = "findByAll" + cls;
		Object[] res = (Object[]) getCache(key);
		if (res == null) {
			res = DBManager.findByAll(cls);
			registCache(key, res); // TODO clone した方が良いかも
		}
		return res;
	}

}
