package jp.co.powerbeans.vbscore.logic;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import jp.co.powerbeans.vbscore.MsgSwt;
import jp.co.powerbeans.vbscore.db.DBCacheManager;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.db.DBUtil;
import jp.co.powerbeans.vbscore.logic.ranking.RankingAnalyzer;
import jp.co.powerbeans.vbscore.logic.ranking.RankingAnalyzerFactory;
import jp.co.powerbeans.vbscore.model.ChangePlayer;
import jp.co.powerbeans.vbscore.model.Game;
import jp.co.powerbeans.vbscore.model.GameSet;
import jp.co.powerbeans.vbscore.model.GameTeams;
import jp.co.powerbeans.vbscore.model.Play;
import jp.co.powerbeans.vbscore.model.Player;
import jp.co.powerbeans.vbscore.model.PlayerRanking;
import jp.co.powerbeans.vbscore.model.RankingDef;
import jp.co.powerbeans.vbscore.model.TeamAnalyze;
import jp.co.powerbeans.vbscore.model.TeamParGameAnalyze;

import org.apache.commons.beanutils.BeanUtils;


/**
 * スコア関連の計算・検索を行うロジック
 * @author amonden
 */
public final class ScoreMgr {

  /**
   * gameId の終了済みセット数
   * @param gameId ゲームID
   * @return 終了済みセット数
   */
  public static int getFinishedSetCount(int gameId) {
    
    return DBManager.countBy(GameSet.class, 
        " gameid = " + gameId +
        " AND winTeamId > 0 ");
  }
  /**
   * gameId で teamId が勝利したセット数
   * @param gameId ゲームID
   * @return 勝利セット数
   */
  public static int getSetWinCount(int gameId, int teamId) {
    
    return DBManager.countBy(GameSet.class, 
        " gameid = " + gameId +
        " AND winTeamId = " + teamId);
  }

	/**
	 * gameId の終了済みGameSetを取得
	 * @param gameId ゲームID
	 * @return 終了済みセット
	 */
	public static GameSet[] getFinishedSets(int gameId) {
		
		return (GameSet[]) DBManager.findBy(GameSet.class, 
				" gameid = " + gameId +
				" AND winTeamId > 0 ",
				" gamesetid ");
	}

	/**
	 * gameId の最終GameSetを取得
	 * @param gameId gameId
	 * @return 最終GameSet
	 */
	public static GameSet getLastSet(int gameId) {
		
		return (GameSet)DBManager.find1By(GameSet.class,
			" gameSetId = (" +
			"  select MAX(gameSetId) from gameset where gameId = " + gameId +
			" )" );
	}

//	public static List getPointList(int gameSetId) {
//		return Arrays.asList(DBManager.findBy(Point.class, "gamesetid = " + gameSetId, "pointId DESC"));
//	}
//
//	public static List getPointList(int gameSetId, int teamId) {
//		return Arrays.asList(DBManager.findBy(Point.class, "gamesetid = " + gameSetId
//				 + " AND actionPlayerTeamId = " + teamId, "pointId DESC"));
//	}

	/**
	 * gameSetId, teamId の全Playを playId 降順に取得
	 * @param gameSetId gameSetId
	 * @param teamId チームID
	 * @return Playリスト
	 */
	public static List getPlayList(int gameSetId, int teamId) {
		return Arrays.asList(DBManager.findBy(Play.class, "gamesetid = " + gameSetId
				 + " AND actionPlayerTeamId = " + teamId, "playId DESC"));
	}
	
	/**
	 * gameSetId の 全Playを playId 降順に取得
	 * @param gameSetId gameSetId
	 * @return Playリスト
	 */
	public static List getPlayList(int gameSetId) {
		return Arrays.asList(DBManager.findBy(Play.class, "gamesetid = " + gameSetId, "playId DESC"));
	}
	
	/**
	 * myps の propname プロパティ値を set の propname + (配列インデックス) にコピー
	 * @param myps コピー元Player情報
	 * @param propname プロパティ名
	 * @param set コピー先Set
	 */
	public static void copyPlayerToSet(Player[] myps, String propname, GameSet set) {
		for (int i = 0; i < myps.length; i++) {
			try {
				BeanUtils.copyProperty(set, propname + (i + 1), new Integer(myps[i].getPlayerId()));
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * set の myPlayingPlayerId[i] プロパティの各値を Player にコピーしてリストで返す
	 * @param set コピー元Set
	 * @param vsDummyPlayer true ダミープレイヤーを追加する, false 追加しない
	 * @param vsTeamId vsDummyPlayer = true の場合 追加するvsTeamId
	 * @return コピーして生成したPlayer配列
	 */
	public static Player[] copySetToMyPlayingPlayers(GameSet set, boolean vsDummyPlayer, int vsTeamId) {
		List<Object> list = new ArrayList<Object>();
		for (int i = 0; i < 7; i++) {
			try {
				int id = Integer.parseInt(BeanUtils.getProperty(set, "myPlayingPlayerId" + (i + 1)));
				if (id > 0) {
					list.add(DBManager.findByPrimaryKey(Player.class, id));
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		if (vsDummyPlayer) {
			list.add(null);
			list.add(createVsTeamDummyPlayer(vsTeamId));
		}
		
		return (Player[]) list.toArray(new Player[0]);
	}

	/**
	 * 対戦チームのダミープレイヤーを生成
	 * @param team_id チームID
	 * @return ダミープレイヤー
	 */
	private static Player createVsTeamDummyPlayer(int team_id) {
		Player dp = new Player();
		dp.setPlayerId(VBConst.DUMMY_PLAYER_ID);
		dp.setPlayerName("(対戦相手)");
		dp.setPosit("");
		dp.setStdNumber("-1");
		dp.setTeamId(team_id);

		return dp;
	}

	/**
	 * Play.getActionPlayerId() をキーとしてPlayerをキャッシュから取得
	 * @param p Play
	 * @return Player
	 */
	public static Player getPlayerCacheSafe(Play p) {
		if (p.getActionPlayerId() == VBConst.DUMMY_PLAYER_ID) {
			return createVsTeamDummyPlayer(p.getActionPlayerTeamId());
		} else {
			return (Player)DBCacheManager.findByPrimaryKey(Player.class, p.getActionPlayerId());
		}
	}
	
	/**
	 * Game のステータスを完了にしupdateにする
	 * @param game game
	 */
	public static void completeGame(Game game) {
		// TODO セットポイント取得数から勝利チームを取得して
		// 試合を終了ステータスにする
		// セットポイント取得数が同じ場合はconfirm
		
		game.setGameStatus(Game.STATUS_END);
		
		DBManager.update(game);
	}

	/**
	 * set のステータスをセット間にする.
	 * 通常セット終了時にコールする
	 * @param game game
	 * @param set set
	 */
	public static void completeSet(Game game, GameSet set) {

		if (set.getMyResultScore() == set.getVsResultScore()) {
			// セットポイントが同じです
			set.setWinTeamId(MsgSwt.isMyTeamWin() ? game.getMyTeamId() : game
					.getVsTeamId());
		} else {
			set.setWinTeamId(set.getMyResultScore() > set
							.getVsResultScore() ? game.getMyTeamId() : game
							.getVsTeamId());
		}
		
		game.setGameStatus(Game.STATUS_BETWEEN_SET);
		DBManager.update(set);
		DBManager.update(game);
	}

	/**
	 * Setをgameから作成
	 * 
	 * @param game Game
	 * @param num 
	 * @return 生成したSet
	 */
	public static GameSet createGameSet(Game game, int num) {
		GameSet set = new GameSet();
		set.setGameId(game.getGameId());
		set.setSetNum(num);
		
		return set;
	}

	/**
	 * teamId のプレイヤーのうち, playing_players に所属しないプレイヤー配列を返す
	 * @param teamId チームID
	 * @param playing_players 対象外プレイヤー配列
	 * @return playing_players 以外のプレイヤー配列
	 */
	public static Player[] getSubPlayers(int teamId, Player[] playing_players) {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < playing_players.length; i++) {
			if (i > 0) {
				buf.append(",");
			}
			buf.append(playing_players[i].getPlayerId());
		}
		

		return (Player[]) DBManager.findBy(Player.class, " teamId = " + teamId + " AND playerID NOT IN(" + buf.toString() + ")");
	}

	/**
	 * gameSetId の選手交代情報を取得
	 * @param gameSetId gameSetId
	 * @return 選手交代情報
	 */
	public static List getChangePlayerList(int gameSetId) {
		return Arrays.asList(DBManager.findBy(ChangePlayer.class, "gamesetid = " + gameSetId, "changePlayerId DESC"));
	}

	/**
	 * チーム毎のプレー集計
	 * @param team_id チームID
	 * @param action_id アクションID
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 集計結果
	 */
	public static TeamAnalyze[] analayzeSearchTeam(int team_id, Date from_dt, Date to_dt) {
		StringBuffer sub = new StringBuffer();
		sub.append(DBUtil.sqlDateBetween("G.gameDate", from_dt, to_dt));
		
		StringBuffer sql = new StringBuffer( 
			"SELECT 0 playerID, '(チーム)' name, p.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultName, count(*) times, psum.alltimes, 100 * count(*) / alltimes per " + 
				"FROM Game G, GameSet S, Play p, ActionDef ad, ActionResultDef ar " +  
				",(SELECT actionDefId, count(*) alltimes FROM Play, Game, GameSet " +  
				"   WHERE Game.gameId = GameSet.gameId AND GameSet.gamesetId = Play.gamesetId AND actionPlayerTeamId = " + team_id + " GROUP BY actionDefId ) psum " + 
				" WHERE G.gameId = S.gameId  " +
				sub.toString() +
				" AND S.gamesetid = p.gamesetid   " + 
				" AND p.actionPlayerTeamId = " + team_id + 
				" AND p.actionDefId = ad.actionDefId " +  
				" AND p.actionDefId = psum.actionDefId " + 
				" AND p.actionResultDefId = ar.actionResultDefId " +  
				" group by p.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultDefId, ar.actionResultName ,  psum.alltimes " + 
				" ORDER BY p.actionDefId, ar.actionResultDefId,  times");
		
		TeamAnalyze[] ta = (TeamAnalyze[]) DBManager.find(TeamAnalyze.class, sql.toString());
		// データ加工 前行と同じアクション名は空とする
		DBUtil.clearDuplicate(ta, "actionName");
		DBUtil.clearDuplicate(ta, "name");
		
		return ta;
	}

	/**
	 * チーム・プレイヤー毎のプレー集計
	 * @param team_id チームID
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 集計結果
	 */
	public static TeamAnalyze[] analayzeSearchPlayers(int team_id, Date from_dt, Date to_dt) {
		StringBuffer sub = new StringBuffer();
		sub.append(DBUtil.sqlDateBetween("G.gameDate", from_dt, to_dt));
		
		StringBuffer sql = new StringBuffer( 
			"SELECT pl.playerId, pl.playerName name, p.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultName, count(*) times, psum.alltimes, 100 * count(*) / alltimes per " + 
				"FROM Game G, GameSet S, Play p, ActionDef ad, ActionResultDef ar, Player pl " +  
				",(SELECT actionDefId, count(*) alltimes FROM Play, Game, GameSet " +  
				"   WHERE Game.gameId = GameSet.gameId AND GameSet.gamesetId = Play.gamesetId AND actionPlayerTeamId = " + team_id + " GROUP BY actionDefId ) psum " + 
				" WHERE G.gameId = S.gameId  " +
				sub.toString() +
				" AND S.gamesetid = p.gamesetid   " + 
				" AND p.actionPlayerTeamId = " + team_id + 
				" AND p.actionPlayerId = pl.playerId " +
				" AND p.actionDefId = ad.actionDefId " +  
				" AND p.actionDefId = psum.actionDefId " + 
				" AND p.actionResultDefId = ar.actionResultDefId " +  
				" group by pl.playerId, pl.playerName, p.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultDefId, ar.actionResultName ,  psum.alltimes " + 
				" ORDER BY pl.playerId, ar.actionResultDefId,  times");
		
		TeamAnalyze[] ta = (TeamAnalyze[]) DBManager.find(TeamAnalyze.class, sql.toString());
		// データ加工 前行と同じアクション名,氏名は空とする
		DBUtil.clearDuplicate(ta, "actionName");
		DBUtil.clearDuplicate(ta, "name");
		
		return ta;
	}

	/**
	 * チーム・プレイヤー・セット毎のプレー集計
	 * @param team_id チームID
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 集計結果
	 */
	public static TeamParGameAnalyze[] analyzeSearchTeamPerGameSet(int team_id, Date from_dt, Date to_dt) {
		StringBuffer sub = new StringBuffer();
		sub.append(DBUtil.sqlDateBetween("G.gameDate", from_dt, to_dt));
		
//		{"gameId", "gameDate", "compeName", "gameName", "vsTeamName", "setNum", "actionName", "actionResultName", "times", "per"}
		
		StringBuffer sql = new StringBuffer(
			"SELECT G.gameId, G.gameDate, G.compeName, G.gameName, vt.teamName vsTeamName, S.setNum, ad.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultName, count(*) times, psum.alltimes, 100 * count(*) / alltimes per " + 
				"FROM Game G, GameSet S, Play p, ActionDef ad, ActionResultDef ar, Team vt " +  
				",(SELECT actionDefId, count(*) alltimes FROM Play, Game, GameSet " +  
				"   WHERE Game.gameId = GameSet.gameId AND GameSet.gamesetId = Play.gamesetId AND actionPlayerTeamId = " + team_id + " GROUP BY actionDefId ) psum " + 
				" WHERE G.gameId = S.gameId  " +
				" AND G.vsTeamId = vt.teamId " +
				sub.toString() +
				" AND S.gamesetid = p.gamesetid   " + 
				" AND p.actionPlayerTeamId = " + team_id + 
				" AND p.actionDefId = ad.actionDefId " +  
				" AND p.actionDefId = psum.actionDefId " + 
				" AND p.actionResultDefId = ar.actionResultDefId " +  
				" group by G.gameId, G.gameDate, G.compeName, G.gameName, vt.teamName, S.setNum, ad.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultName ,  psum.alltimes ");
		sql.append(
		" UNION ALL " +
		"SELECT G.gameId, G.gameDate, G.compeName, G.gameName, vt.teamName vsTeamName, 0 setNum, ad.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultName, count(*) times, psum.alltimes, 100 * count(*) / alltimes per " + 
		"FROM Game G, GameSet S, Play p, ActionDef ad, ActionResultDef ar, Team vt " +  
		",(SELECT actionDefId, count(*) alltimes FROM Play, Game, GameSet " +  
		"   WHERE Game.gameId = GameSet.gameId AND GameSet.gamesetId = Play.gamesetId AND actionPlayerTeamId = " + team_id + " GROUP BY actionDefId ) psum " + 
		" WHERE G.gameId = S.gameId  " +
		" AND G.vsTeamId = vt.teamId " +
		sub.toString() +
		" AND S.gamesetid = p.gamesetid   " + 
		" AND p.actionPlayerTeamId = " + team_id + 
		" AND p.actionDefId = ad.actionDefId " +  
		" AND p.actionDefId = psum.actionDefId " + 
		" AND p.actionResultDefId = ar.actionResultDefId " +  
		" group by G.gameId, G.gameDate, G.compeName, G.gameName, vt.teamName, ad.actionDefId, ad.actionName, ar.actionResultDefId, ar.actionResultName ,  psum.alltimes ");
		
//		TODO UNION で全セットの集計も作成する
		sql.append(" ORDER BY gameDate, gameId, setNum, actionDefId, actionResultDefId");
		
		TeamParGameAnalyze[] ta = (TeamParGameAnalyze[]) DBManager.find(TeamParGameAnalyze.class, sql.toString());
		// データ加工 前行と同じアクション名は空とする
		DBUtil.clearDuplicate(ta, "compeName");
		DBUtil.clearDuplicate(ta, "gameName");
		DBUtil.clearDuplicate(ta, "gameId");
		DBUtil.clearDuplicate(ta, "gameDate");
		DBUtil.clearDuplicate(ta, "vsTeamName");
//		DBUtil.clearDuplicate(ta, "setNum");
//		DBUtil.clearDuplicate(ta, "actionName");
		
		// 0set を '全セット' に変更
		DBUtil.replace(ta, "setNum", "0", "全セット");
		
		return ta;
	}

	/**
	 * team_id チーム内での各ランキング集計結果
	 * @param team_id チームid
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 集計結果( [0] = "サービスエース率"の集計結果, [1] = "サーブミス率"の集計結果 (以下RankingDefに従う)
	 */
	public static PlayerRanking[][] analyzePlayerRanking(int team_id, Date from_dt, Date to_dt) {

		// 1. ランキングの種類を取得
		RankingDef[] ranking_defs = (RankingDef[]) DBManager.findByAll(RankingDef.class);

		// 2. 各ランキング毎に計算
		PlayerRanking[][] allrank = new PlayerRanking[ranking_defs.length][];
		for (int i = 0; i < ranking_defs.length; i++) {
			RankingAnalyzer mgr = RankingAnalyzerFactory.createInstance(ranking_defs[i]);
			allrank[i] = mgr.analyze(team_id, from_dt, to_dt);
		}
		
		return allrank;
	}

	/**
	 * 試合リストを取得
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 試合配列
	 */
	public static Game[] getGameList(Date from_dt, Date to_dt) {
		
		StringBuffer sub = new StringBuffer();
		sub.append(DBUtil.sqlDateBetween("gameDate", from_dt, to_dt));

		return (Game[]) DBManager.findBy(Game.class, sub.toString(), " gameDate, compeName, gameName ");
	}

	/**
	 * 試合リストを試合チームリストとして取得
	 * @param myteam_id 自チーム
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 試合チーム配列
	 */
	public static GameTeams[] getGameListToGameTeams(int myteam_id, Date from_dt, Date to_dt) {
		
		StringBuffer sub = new StringBuffer(GameTeams.SQL);
		sub.append(" AND MT.teamId = " + myteam_id);
		sub.append(DBUtil.sqlDateBetween("gameDate", from_dt, to_dt));
		sub.append(" ORDER BY gameDate, compeName, gameName ");

		return (GameTeams[]) DBManager.find(GameTeams.class, sub.toString());
	}
}
