package jp.co.powerbeans.vbscore.logic.ranking;

import java.util.Date;

import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.db.DBUtil;
import jp.co.powerbeans.vbscore.model.Game;
import jp.co.powerbeans.vbscore.model.PlayerRanking;
import jp.co.powerbeans.vbscore.model.RankingDef;

/**
 * 分析検索Supportクラス
 * @author amonden
 */
public abstract class AnalyzerBase implements RankingAnalyzer {

	/**
	 * 分析対象ランキング
	 */
	private RankingDef rankingDef;

	/**
	 * コンストラクタ
	 * @param ranking_def 分析ランキング
	 */
	public AnalyzerBase(RankingDef ranking_def) {
		super();
		this.rankingDef = ranking_def;
	}

	/**
	 * 分析対象ランキング取得
	 * @return rankingDef を戻します。
	 */
	public RankingDef getRankingDef() {
		return rankingDef;
	}

	/**
	 * 様々なランキングを集計する、汎用ランキング集計メソッド.
	 * @param team_id チームID
	 * @param from_dt 集計開始日
	 * @param to_dt 集計終了日
	 * @param action_def_id アクションID
	 * @param action_result_def_type 対象アクション結果ID
	 * @return プレイヤーランキング
	 */
	protected PlayerRanking[] analyze(int team_id, Date from_dt, Date to_dt, int action_def_id, int action_result_def_type) {
		return analyze(team_id, null, from_dt, to_dt, action_def_id, new int[]{action_result_def_type});
	}

	/**
	 * 様々なランキングを集計する、汎用ランキング集計メソッド.
	 * @param team_id チームID
	 * @param from_dt 集計開始日
	 * @param to_dt 集計終了日
	 * @param action_def_id アクションID
	 * @param action_result_def_type 対象アクション結果ID
	 * @return プレイヤーランキング
	 */
	protected PlayerRanking[] analyze(int team_id, Game[] games, Date from_dt, Date to_dt, int action_def_id, int action_result_def_type) {
		return analyze(team_id, games, from_dt, to_dt, action_def_id, new int[]{action_result_def_type});
	}

	/**
	 * 様々なランキングを集計する、汎用ランキング集計メソッド.
	 * @param team_id チームID
	 * @param from_dt 集計開始日
	 * @param to_dt 集計終了日
	 * @param action_def_id アクションID
	 * @param action_result_def_type 対象アクション結果ID
	 * @return プレイヤーランキング
	 */
	protected PlayerRanking[] analyze(int team_id, Game[] games, Date from_dt, Date to_dt, int action_def_id, int[] action_result_def_type) {
		StringBuffer sub = new StringBuffer();
		sub.append(DBUtil.sqlDateBetween("G.gameDate", from_dt, to_dt));
		String types = DBUtil.sqlToIn(action_result_def_type);
		String gameids = DBUtil.sqlToIn(games, "gameId", "G.gameId");
		
		StringBuffer sql = new StringBuffer(
				" SELECT pl.playerId, pl.playerName name, 1 rankingDefId, '" + getRankingDef().getRankingDefId()  + "', " + 
				"  '" + getRankingDef().getRankingName() + "' rankingName, count(*) times, count(*) + '回' timesName, psum.alltimes, (100 * count(*) / psum.alltimes) percent," +
						"(100 * count(*) / psum.alltimes) + '%' pser  " + 
				"  FROM Game G, GameSet S, Play p, ActionDef ad, ActionResultDef ar, Player pl , " + 
				"        (SELECT Play.actionPlayerId, count(*) alltimes FROM Play, Game G, GameSet    " + 
				"           WHERE G.gameId = GameSet.gameId AND GameSet.gamesetId = Play.gamesetId  " + gameids + sub.toString() + 
				"           AND actionPlayerTeamId = " + team_id + " AND actionDefId = " + action_def_id +
				"           GROUP BY Play.actionPlayerId ) psum " +  
				" WHERE G.gameId = S.gameId " + gameids + sub.toString() +
				"   AND S.gamesetid = p.gamesetid    AND p.actionPlayerTeamId = " + team_id + 
				"     AND p.actionPlayerId = pl.playerId  AND p.actionDefId = " + action_def_id +  sub.toString() +
				"     AND p.actionDefId = ad.actionDefId " + 
				"     AND p.actionResultDefId = ar.actionResultDefId " +  
				"     AND ar.type IN " + types +
				"     AND p.actionPlayerId = psum.actionPlayerId " + 
				" GROUP BY pl.playerId, pl.playerName, rankingDefId, rankingName, alltimes " +
				" ORDER BY percent DESC, times DESC"
				);
	
		PlayerRanking[] pls = (PlayerRanking[]) DBManager.find(PlayerRanking.class, sql.toString());
		return pls;
	}

	
}
