package jp.co.powerbeans.vbscore.swt;

import jp.co.powerbeans.common.swt.ChkConvSwt;
import jp.co.powerbeans.vbscore.CMD;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.MsgSwt;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.logic.ScoreMgr;
import jp.co.powerbeans.vbscore.model.Game;
import jp.co.powerbeans.vbscore.model.GameSet;
import jp.co.powerbeans.vbscore.model.GameTeams;
import jp.co.powerbeans.vbscore.model.Play;
import jp.co.powerbeans.vbscore.model.Team;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

/**
 * 試合一覧画面
 * @author amonden
 */
public class GameListSwt extends PBSwt {

	/**
	 * コンポジット
	 */
	private Composite comp;
	/**
	 * 一覧table
	 */
	private Table table;

	/**
	 * ウィンドウ幅
	 */
	static final int WINDOW_WIDTH = 600;
	/**
	 * ウィンドウ高さ
	 */
	static final int WINDOW_HEIGTH = 400;

	/**
	 * タイトル
	 */
	private static final String TITLE = Msg.getGameList();
	/**
	 * トップレベルメニューラベル
	 */
	private static final String[] MENU_LABELS = {Msg.getMenuEdit(), Msg.getMenuDel()};
	/**
	 * トップレベルメニューコマンド
	 */
	private static final String[] MENU_CMDS = {CMD.EDIT, CMD.DEL};
	/**
	 * table列名
	 */
	private static final String[] TABLE_COLS = {Msg.getId(), Msg.getDate(), Msg.getCompeName(), Msg.getGameName(), Msg.getOwnTeam(), Msg.getTargetVs(), Msg.getStatus()};
	/**
	 * table表示プロパティ名
	 */
	private static final String[] TABLE_MODEL_COLS = new String[]{"gameId", "gameDate", "compeName", "gameName", "myTeamId_teamName", "vsTeamId_teamName", "statusName"};
	/**
	 * table列数
	 */
	private static final int[] TABLE_COL_WIDTHS = {30,100, 100,100, 100, 80, 60};

	/**
	 * コンストラクタ
	 */
	public GameListSwt() {
		super();
		initialize();
	}	
	
	/**
	 * 初期化
	 */
	private void initialize() {
		createSShell(TITLE, WINDOW_WIDTH, WINDOW_HEIGTH, true);
		setGridLayout(1);
		createComposit();
		createItems();
		createMenu();
	}

	/**
	 * 全項目生成
	 */
	private void createItems() {
		createTable();
	}

	/**
	 * table生成
	 */
	private void createTable() {
		if (table == null) {
			table = addSimpleTable(TABLE_COLS, TABLE_COL_WIDTHS, 200);
			updateTable();
			setCmdToTableOnDoubleClick(this.table, CMD.EDIT);
			setHeight(table, WINDOW_HEIGTH);
		}
	}

	/**
	 * 編集メニュー選択時
	 */
	public void cmdEdit() {
		int id = ChkConvSwt.getSelectId(this.table);
		// TODO 状態によりどの画面に遷移するか判断する
		Game game = (Game) DBManager.findByPrimaryKey(Game.class, id);
		GameSet lastset = ScoreMgr.getLastSet(game.getGameId());
		
		switch (game.getGameStatus()) {
		case Game.STATUS_PREPARE:
			new OfficialGameSetStartSwt(game, lastset).show();
			break;
		case Game.STATUS_PLAYING:
			new OfficialGameInputSwt(game, lastset).show();
			break;
		case Game.STATUS_BETWEEN_SET:
			new OfficialGameSetStartSwt(game, lastset).show();
			break;
		case Game.STATUS_END:
			break;

		default:
			break;
		}
		updateTable();
	}
	
	/**
	 * 削除メニュー選択時
	 */
	public void cmdDel() {
		deleteTable();
	}

	/**
	 * tableの選択行を削除
	 */
	private void deleteTable() {
		int[] ids = ChkConvSwt.getSelectIds(this.table);
		if (MsgSwt.isDelete()) {
			for (int i = 0; i < ids.length; i++) {
				DBManager.removeByPrimaryKey(Game.class, ids[i]);
				GameSet[] gamesets = (GameSet[]) DBManager.findBy(GameSet.class, " gameId = " + ids[i]);
				for (int j = 0; j < gamesets.length; j++) {
					DBManager.removeBy(Play.class, " gameSetId = " + gamesets[j].getGameSetId());
				}
				DBManager.removeBy(GameSet.class, " gameId = " + ids[i]);
			}
			updateTable();
		}
	}

	/**
	 * tableを最新データで更新
	 */
	private void updateTable() {
		this.table.removeAll();
		
		DBManager.findByAllToSwtTable(this.table, GameTeams.class, GameTeams.SQL, TABLE_MODEL_COLS, new Class[]{Game.class, Team.class});
		// いつかこうしたい
//		DBManager.findByAllToSwtTable(this.table, GameTeams.class, new Class[]{Game.class, Team.class, Team.class}, TABLE_MODEL_COLS);
	}

	/**
	 * コンポジット生成
	 */
	private void createComposit() {
		if (this.comp == null) {
		    this.comp = createNRow1ColComposit(); 
		}
	}

	/**
	 * メニュー生成
	 */
	private void createMenu() {
        create1LayerMenu(MENU_LABELS, MENU_CMDS);
	}
}
