package jp.co.powerbeans.vbscore.swt;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import jp.co.powerbeans.common.swt.ChkConvSwt;
import jp.co.powerbeans.common.swt.CmdRSingleClickMouseListener;
import jp.co.powerbeans.common.swt.CmdSelectionAdapter;
import jp.co.powerbeans.vbscore.CMD;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.MsgSwt;
import jp.co.powerbeans.vbscore.db.DBCacheManager;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.logic.LicenseMgr;
import jp.co.powerbeans.vbscore.logic.ScoreMgr;
import jp.co.powerbeans.vbscore.model.ActionDef;
import jp.co.powerbeans.vbscore.model.ActionResultDef;
import jp.co.powerbeans.vbscore.model.ChangePlayer;
import jp.co.powerbeans.vbscore.model.Game;
import jp.co.powerbeans.vbscore.model.GameSet;
import jp.co.powerbeans.vbscore.model.Play;
import jp.co.powerbeans.vbscore.model.Player;
import jp.co.powerbeans.vbscore.model.Team;
import jp.co.powerbeans.vbscore.swt.parts.InputPlayComposit;
import jp.co.powerbeans.vbscore.swt.parts.InputPlayCompositEventListener;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;

/**
 * 公式戦入力画面
 * @author amonden
 * 
 */
public class OfficialGameInputSwt extends PBSwt implements InputPlayCompositEventListener {

	/**
	 * 入力中の試合
	 */
	private Game game;

	/**
	 * 入力中のセット
	 */
	private GameSet set;

	/**
	 * 自チーム
	 */
	private Team myTeam;

	/**
	 * 対戦チーム
	 */
	private Team vsTeam;

	/**
	 * セット数、スコア表示テーブル
	 */
	private Table scoreTable;

	/**
	 * 自チームセットポイント
	 */
	private int myTeamSetCount;
	
	/**
	 * 対戦チームセットポイント
	 */
	private int vsTeamSetCount;

	/**
	 * 以前のセット
	 */
	private GameSet[] beforeSets;

	/**
	 * プレー詳細テーブル
	 */
	private Table playDtlTable;

	/**
	 * プレーリスト.プレー詳細テーブルの元データ
	 */
	private List playList;
	
	/**
	 * 選手交代情報
	 */
	private List changePlayerList;

	/**
	 * プレー入力パネル
	 */
	private InputPlayComposit inputPanel;
	
	/**
	 * ウィンドウ幅
	 */
	static final int WINDOW_WIDTH = 800;

	/**
	 * ウィンドウ高さ
	 */
	static final int WINDOW_HEIGTH = 600;

	/**
	 * タイトル
	 */
	private static final String TITLE = Msg.getOfficialGameHfun();

	/**
	 * トップレベルメニューラベル
	 */
	private static final String[] MENU_LABELS = {Msg.getMenuPause(), Msg.getMenuExitSet(), Msg.getMenuChangePlayer()};
	/**
	 * トップレベルメニューコマンド
	 */
	private static final String[] MENU_CMDS = {CMD.EXIT, CMD.SET_COMPLETE, CMD.CHANGE_PLAYER};
	
	/**
	 * プレー詳細テーブルポップメニューラベル
	 */
	private static final String[] MENU_PLAY_POP_LABELS = {Msg.getMenuDel()};
	/**
	 * プレー詳細テーブルポップメニューコマンド
	 */
	private static final String[] MENU_PLAY_POP_CMDS = {CMD.DEL};

	/**
	 * コンストラクタ
	 * @param game 対象試合
	 * @param set 対象セット
	 */
	public OfficialGameInputSwt(Game game, GameSet set) {
		super();
		this.game = game;
		this.set = set;
		initialize();
	}

	/**
	 * 初期化
	 */
	private void initialize() {
		loadData();
		createSShell(TITLE + this.game.getCompeName() + "-" + this.game.getGameName() + " " +
				Msg.getThe() + this.set.getSetNum() + Msg.getSet(),
				WINDOW_WIDTH, WINDOW_HEIGTH, true);
		setGridLayout(3);
//		createComposit();
		createMenu();
		createItems();
		
		updateTables();
	}

	/**
	 * メニュー生成
	 */
	private void createMenu() {
        create1LayerMenu(MENU_LABELS, MENU_CMDS);
	}
	
	/**
	 * 各データロード
	 */
	private void loadData() {
		this.myTeam = (Team) DBManager.findByPrimaryKey(Team.class, this.game.getMyTeamId());
		this.vsTeam = (Team) DBManager.findByPrimaryKey(Team.class, this.game.getVsTeamId());
		// セットカウントを取得
		this.myTeamSetCount = ScoreMgr.getSetWinCount(this.game.getGameId(), this.myTeam.getTeamId());
		this.vsTeamSetCount = ScoreMgr.getSetWinCount(this.game.getGameId(), this.vsTeam.getTeamId());
		// 以前までのセット数を取得
		this.beforeSets = ScoreMgr.getFinishedSets(this.game.getGameId());
		
		// プレイリストロード
		loadPlayList();
		
		// 選手交代ロード
		this.changePlayerList = new ArrayList(ScoreMgr.getChangePlayerList(this.set.getGameSetId()));
	}

	/**
	 * 選手リストロード
	 */
	private void loadPlayList() {
		this.playList = new ArrayList(ScoreMgr.getPlayList(this.set.getGameSetId()));
	}

	/**
	 * 各項目を生成
	 */
	private void createItems() {
		setWidth(addLabel_left(""), 100);
		setWidth(addLabel_left(""), 300);
		setWidth(addLabel_left(""), 300);
		
		// プレー入力パネル
		this.inputPanel = new InputPlayComposit(this.sShell, this.game, this.set, this.myTeam, this);
		setVerticalSpan(this.inputPanel, 3);
		
		// ポイントテーブル
		this.scoreTable = addSimpleTable(new String[]{"", this.myTeam.getTeamName(), this.vsTeam.getTeamName()}, new int[]{100,100,100}, 200);
		setHorizontalSpan(this.scoreTable, 2);

		// プレー詳細テーブル
		this.playDtlTable = addSimpleTable(new String[]{
				Msg.getSetPts(), Msg.getPoint(), Msg.getOwnPlayer(), Msg.getLastPlay(), "",
				Msg.getSetPts(), Msg.getPoint(), Msg.getVsPlayer(), Msg.getLastPlay()}, 
				new int[]{50,40,90,140, 20, 50,40,90,140}, 200);
		setHorizontalSpan(this.playDtlTable, 2);
		setVerticalSpan(this.inputPanel, 2);
//		setCmdToTableOnSingleClick(this.playDtlTable, CMD.SEL1ROW);
		this.playDtlTable.addMouseListener(new CmdRSingleClickMouseListener(this, CMD.SEL1ROW));
		
		// TODO コメント入力欄追加
		super.sShell.addKeyListener(new InputPlayKeyListener());
	}
	
	/**
	 * セット終了選択時
	 */
	public void cmdExit() {
		super.cmdOk();
	}
	
	/**
	 * プレー詳細テーブルで右クリック選択時
	 */
	public void cmdSel1row() {
		// ポップメニューを表示
		Menu menu = new Menu(this.sShell);
		
		for (int i = 0; i < MENU_PLAY_POP_LABELS.length; i++) {
			MenuItem menuItem0 = new MenuItem(menu, SWT.PUSH);
			menuItem0.setText(MENU_PLAY_POP_LABELS[i]);
//			menuItem0.setData(selIndex);
 			menuItem0.addSelectionListener(new CmdSelectionAdapter(this, MENU_PLAY_POP_CMDS[i]));
		}
	
		this.sShell.setMenu(menu);
		menu.setVisible(true);
	}
	
	/**
	 * 削除メニュー選択時
	 */
	public void cmdDel() {
		// 1. 確認メッセージ
    
    Play[] plays = (Play[]) ChkConvSwt.getSelectObjects(this.playDtlTable);
//		int[] ids = ChkConvSwt.getSelectIds(this.playDtlTable);
		if (plays.length > 0 && MsgSwt.isDelete()) {
			// 2. 削除 本来は1トランザクションで
			
			// プレーを削除
			int myTeamSetPts = 0; // 自チームのマイナス点数
			int vsTeamSetPts = 0; // 相手チームのマイナス点数
			
			for (int i = 0; i < plays.length; i++) {
				// DBから削除
				DBManager.removeByPrimaryKey(Play.class, plays[i].getPlayId());
			}
			
			// Play再ロード
			loadPlayList();
			// playListから現在までの点数を再集計
			for (Iterator it = this.playList.iterator(); it.hasNext();) {
				Play p = (Play) it.next();
				myTeamSetPts += p.getMyTeamAddPoint();
				vsTeamSetPts += p.getVsTeamAddPoint();
			}
			
			// セットの点数を更新
			GameSet set = (GameSet) DBManager.findByPrimaryKey(GameSet.class, this.set.getGameSetId());
			set.setMyResultScore(myTeamSetPts);
			set.setVsResultScore(vsTeamSetPts);
			DBManager.update(set);
			this.set = set;
			
			// 3. 更新
			updatePlayDtlTable();
			updateScoreTable();
		}
	}
	
	/**
	 * セット終了メニュー選択時
	 */
	public void cmdSet_complete() {
		// 1. 勝利チームの確認
		ScoreMgr.completeSet(this.game, this.set);
		
		// 2. 次のセットを入力するか確認
		if (!MsgSwt.isGotoNextSet()) {
			// 試合終了へ
			ScoreMgr.completeGame(this.game);
			cmdOk();
			return;
		}
		
		// 3. 次のセットへ
		GameSet next_set = ScoreMgr.createGameSet(game, this.set.getSetNum() + 1);
		// set を insert
		DBManager.insert(next_set);
		// セット開始画面へ
    cmdOk();
		new OfficialGameSetStartSwt(this.game, next_set).show();
	}
	
	/**
	 * 選手交代メニュー選択時
	 */
	public void cmdChange_player() {
		// 選手交代ダイアログ
		// 現在は自チームのみ
		Player[] old_myplayers = ScoreMgr.copySetToMyPlayingPlayers(this.set, false, this.myTeam.getTeamId());
		MemberChangeSwt swt = new MemberChangeSwt(this.myTeam.getTeamId(), old_myplayers);
		swt.show();
		
		if (CMD.isOk(swt.getResultCmd())) {
			Player[] new_myplayers = swt.getNewMyplayers();
			
			// set に保存
			ScoreMgr.copyPlayerToSet(new_myplayers, "myPlayingPlayerId", this.set);
			DBManager.update(this.set);
			
			// 交代履歴を追加
			ChangePlayer cp = new ChangePlayer();
			cp.setGameSetId(this.set.getGameSetId());
			cp.setTeamId(this.myTeam.getTeamId());
			if (this.playList.size() > 0) {
				cp.setLastPlayId(((Play)this.playList.get(this.playList.size() - 1)).getPlayId());
			}
			cp.setInPlayerId(swt.getInPlayerId());
			cp.setOutPlayerId(swt.getOutPlayerId());
			DBManager.insert(cp);
			
			// inputPlayパネル更新
			this.inputPanel.updatePlayingPlayer();
		}
	}

	
	/**
	 * キーリスナー実装.現在未使用
	 * @author amonden
	 *
	 */
	public class InputPlayKeyListener implements KeyListener {

		public void keyPressed(KeyEvent e) {
			if ('1' == e.character) {
				MsgSwt.confirm("1");
			}
		}

		public void keyReleased(KeyEvent e) {
		}

	}
	/* (非 Javadoc)
	 * @see jp.co.powerbeans.vbscore.swt.parts.InputPlayCompositEventListener#addPlay(jp.co.powerbeans.vbscore.model.Play)
	 */
	public void addPlay(Play p) {
		if (this.playDtlTable.getItemCount() >= 3) {
			LicenseMgr.checkLicense();
		}
		
		this.playList.add(0, p);

		if (p.getMyTeamAddPoint() + p.getVsTeamAddPoint() > 0) {
			// ポイントが入った場合はスコアテーブル更新
			updateScoreTable();
		}
		
		addPlayToDtlTable(p);
	}

	/**
	 * 全テーブル更新
	 */
	private void updateTables() {
		updateScoreTable();
		updatePlayDtlTable();
	}

	/**
	 * スコアテーブル更新
	 */
	private void updateScoreTable() {
		this.scoreTable.removeAll();
		
		addTableRow(this.scoreTable, new String[]{Msg.getSetcount(), "" + this.myTeamSetCount, "" + this.vsTeamSetCount});
		for (int i = 0; i < this.beforeSets.length; i++) {
			addTableRow(this.scoreTable, new String[]{Msg.getThe() + (i + 1) + Msg.getSet(), "" + this.beforeSets[i].getMyResultScore(), "" + this.beforeSets[i].getVsResultScore()});
		}
		addTableRow(this.scoreTable, new String[]{Msg.getThe() + (this.beforeSets.length + 1) + Msg.getSet(),
				"" + this.set.getMyResultScore(), "" + this.set.getVsResultScore()} );
	}

	/**
	 * 更新プレー詳細テーブル
	 */
	private void updatePlayDtlTable() {
		this.playDtlTable.removeAll();
		
		for (Iterator it = this.playList.iterator(); it.hasNext();) {
			Play p = (Play)it.next();
			addTableRow(this.playDtlTable,	createPlayDtlTableRowData(p), p );
		}
	}

	/**
	 * プレーからプレー詳細テーブル1行データを生成
	 * @param p プレー
	 * @return プレー詳細テーブル表示用データ
	 */
	private String[] createPlayDtlTableRowData(Play p) {
		Player pl = ScoreMgr.getPlayerCacheSafe(p);
		ActionDef actdef = (ActionDef)DBCacheManager.findByPrimaryKey(ActionDef.class, p.getActionDefId());
		ActionResultDef resdef = (ActionResultDef) DBCacheManager.findByPrimaryKey(ActionResultDef.class, p.getActionResultDefId());
		
		String my_player = "";
		String my_act = "";
		String vs_player = "";
		String vs_act = "";
		
		String player = pl.getStdNumber() + ":" + pl.getPlayerName();
		String act = actdef.getActionName() + ":" + resdef.getActionResultName();
		
		if (p.getActionPlayerTeamId() == this.myTeam.getTeamId()) {
			// プレーが自チームの場合
			my_player = player;
			my_act = act;
		} else {
			vs_player = player;
			vs_act = act;
		}
		return new String[]{
				"" + p.getMyTeamSumPoint(),
				"" + p.getMyTeamAddPoint(),
				my_player,
				my_act,
				"",
				"" + p.getVsTeamSumPoint(),
				"" + p.getVsTeamAddPoint(),
				vs_player,
				vs_act};
	}

	/**
	 * プレーをプレー詳細テーブルに追加
	 * @param p プレー
	 */
	private void addPlayToDtlTable(Play p) {
		insertTableRowTop(this.playDtlTable, createPlayDtlTableRowData(p), p);
	}

//	public void changePlayer(Player[] new_players) {
//		// 画面を更新するだけ
//		updateTables();
//		
//		// 交代履歴に追加
////		DBManager
//	}

}
