package jp.co.powerbeans.vbscore.swt;

import jp.co.powerbeans.common.swt.ChkConvSwt;
import jp.co.powerbeans.common.swt.ChkConvSwtRuntimeException;
import jp.co.powerbeans.vbscore.CMD;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.MsgSwt;
import jp.co.powerbeans.vbscore.db.DBCacheManager;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.model.Player;
import jp.co.powerbeans.vbscore.model.Team;

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

/**
 * プレイヤーリストSwt画面
 * @author amonden
 *
 */
public class PlayerListSwt extends PBSwt {

	/**
	 * コンポジット
	 */
	private Composite comp;
	/**
	 * テーブル
	 */
	private Table table;

	/**
	 * ウィンドウ幅
	 */
	int BUTTON_WIDTH = 400;
	/**
	 * ウィンドウ高さ
	 */
	int BUTTON_HEIGTH = 40;
	/**
	 * チームコンボ
	 */
	private Combo teamCombo;
	/**
	 * チームID
	 */
	private int teamid;
	/**
	 * ウィンドウ幅
	 */
	static final int WINDOW_WIDTH = 420;
	/**
	 * ウィンドウ高さ
	 */
	static final int WINDOW_HEIGTH = 300;

	/**
	 * 高さ
	 */
	private static final String TITLE = Msg.getPlayerList();
	/**
	 * メニュー名
	 */
	private static final String[] MENU_LABELS = {Msg.getMenuEdit(), Msg.getMenuAdd(), Msg.getMenuDel()};
	/**
	 * メニューコマンド
	 */
	private static final String[] MENU_CMDS = {CMD.EDIT, CMD.ADD, CMD.DEL};
	/**
	 * テーブル列名
	 */
	private static final String[] TABLE_COLS = {Msg.getId(), Msg.getFullname(), Msg.getPosition(), Msg.getNumber()};
	/**
	 * テーブル表示プロパティ名
	 */
	private static final String[] TABLE_MODEL_COLS = new String[]{"playerId", "playerName", "posit", "stdNumber"};
	/**
	 * テーブル列幅
	 */
	private static final int[] TABLE_COL_WIDTHS = {50,150, 100,100};

	/**
	 * コンストラクタ
	 */
	public PlayerListSwt() {
		super();
		this.teamid = 0;
		initialize();
	}	
	
	/**
	 * 初期化
	 */
	private void initialize() {
		int team_num = DBManager.countByAll(Team.class);
		if (team_num == 0) {			
			MsgSwt.warn(Msg.getInputBeforeTeam());
			throw new ChkConvSwtRuntimeException("no select combobox.");
		}
		
		createSShell(TITLE, WINDOW_WIDTH, WINDOW_HEIGTH, true);
		setGridLayout(1);
		createComposit();
		createItems();
		createMenu();
		
		// キャッシュクリア
		DBCacheManager.clear();
	}

	/**
	 * 項目生成
	 */
	private void createItems() {
		this.teamCombo = addComboBoxByModel(Team.class, "teamName", CMD.SEL_CHANGE);
		selectTop(this.teamCombo);
		createTable();
	}

	/**
	 * テーブル生成
	 */
	private void createTable() {
		if (table == null) {
			table = addSimpleTable(TABLE_COLS, TABLE_COL_WIDTHS, 200);
			setCmdToTableOnDoubleClick(this.table, CMD.EDIT);
		    cmdSel_change();
		}
	}

	/**
	 * 編集コマンド
	 */
	public void cmdEdit() {
		int id = ChkConvSwt.getSelectId(this.table);
		if (CMD.isOk(new PlayerEditSwt(CMD.EDIT, this.teamid, id).show())) {
			updateTable();
		}
	}
	
	/**
	 * 追加コマンド
	 */
	public void cmdAdd() {
		if (CMD.isOk(new PlayerEditSwt(CMD.ADD, this.teamid, 0).show())) {
			updateTable();
		}
	}
	
	/**
	 * 削除コマンド
	 */
	public void cmdDel() {
		deleteTable();
	}
	
	/**
	 * チームコンボ選択変更時
	 */
	public void cmdSel_change() {
		// 選択後チームIDを取得
		Team team = (Team) ChkConvSwt.getSelectObject(this.teamCombo);
		this.teamid = team.getTeamId();
		updateTable();
	}

	/**
	 * テーブルの選択行を削除
	 */
	private void deleteTable() {
		int[] ids = ChkConvSwt.getSelectIds(this.table);
		if (MsgSwt.isDelete()) {
			for (int i = 0; i < ids.length; i++) {
				DBManager.removeByPrimaryKey(Player.class, ids[i]);
			}
			updateTable();
		}
	}

	/**
	 * テーブルを更新
	 */
	private void updateTable() {
		this.table.removeAll();
		DBManager.findByToSwtTable(this.table, Player.class, TABLE_MODEL_COLS, "teamId = " + this.teamid);
	}

	/**
	 * コンポジット生成
	 */
	private void createComposit() {
		if (this.comp == null) {
		    this.comp = createNRow1ColComposit(); 
		}
	}

	/**
	 * メニュー生成
	 */
	private void createMenu() {
        create1LayerMenu(MENU_LABELS, MENU_CMDS);
	}
}
