package jp.co.powerbeans.vbscore.swt.parts;

import java.util.Date;

import jp.co.powerbeans.common.swt.PBComposite;
import jp.co.powerbeans.common.swt.PBControlUtil;
import jp.co.powerbeans.vbscore.CMD;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.db.DBUtil;
import jp.co.powerbeans.vbscore.logic.ScoreMgr;
import jp.co.powerbeans.vbscore.model.GameTeams;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

/**
 * ミニ試合一覧SwtTableComposit.
 * 
 * @author amonden
 * 
 */
public class GameMiniTableComposit extends PBComposite {

  private static final int COLS = 4;

  private static final String[] TABLE_NAMES = { Msg.getGameDate(), Msg.getCompeName(),
      Msg.getGameName(), Msg.getVsTeam() };

  private static final String[] MODEL_COLS = { "gameDate", "compeName", "gameName",
      "vsTeamId_teamName" };

  private static final int[] WIDS = { 80, 100, 100, 100 };

  private Table table;

  private Button selectAllButton;

  private Button selectClearButton;

  /**
   * コンストラクタ
   * 
   * @param comp
   *          親shell
   * @param colspan
   * @param show_sel_btns
   *          選択ボタン表示フラグ
   */
  public GameMiniTableComposit(Composite comp, int colspan) {
    this(comp, colspan, true);
  }

  /**
   * コンストラクタ
   * 
   * @param comp
   *          親shell
   * @param colspan
   * @param show_sel_btns
   *          選択ボタン表示フラグ
   */
  public GameMiniTableComposit(Composite comp, int colspan, boolean show_sel_btns) {
    super(comp, SWT.NO_FOCUS);

    GridLayout gridLayout = new GridLayout(colspan, true);
    gridLayout.makeColumnsEqualWidth = true;
    setLayout(gridLayout);

    GridData compGrid = new GridData();
    compGrid.horizontalSpan = colspan;
    compGrid.horizontalAlignment = GridData.FILL;
    compGrid.verticalAlignment = GridData.FILL;
    compGrid.grabExcessHorizontalSpace = false;
    // compGrid.grabExcessVerticalSpace = true;
    compGrid.heightHint = 230;
    setLayoutData(compGrid);
    setBackground(new Color(getDisplay(), 255, 255, 255));

    // 対象試合
    PBControlUtil.createLabel_left(this, Msg.getTargetGame());
    if (show_sel_btns) {
      this.selectAllButton = PBControlUtil
          .createSimpleButton(this, Msg.getSelectAll(), CMD.SEL_ALL);
      this.selectClearButton = PBControlUtil.createSimpleButton(this, Msg.getSelectClear(),
          CMD.SEL_CLEAR);
      PBControlUtil.createLabelEmpty(this, colspan - 3);
    } else {
      PBControlUtil.createLabelEmpty(this, 3);
    }

    // create table
    this.table = PBControlUtil.createSimpleTable(this, TABLE_NAMES, WIDS, colspan, 150);
  }

  /**
   * チームIDと日付からリストを更新.
   * 
   * @param team_id
   *          チームID
   * @param from_dt
   *          試合日from
   * @param to_dt
   *          試合日to
   */
  public void updateByDate(int team_id, Date from_dt, Date to_dt) {
    this.table.removeAll();
    GameTeams[] games = ScoreMgr.getGameListToGameTeams(team_id, from_dt, to_dt);
    DBUtil.modelsToSwtTable(this.table, games, MODEL_COLS);
  }

  /**
   * テーブル取得
   * 
   * @return テーブル
   */
  public Table getTable() {
    return table;
  }

  /**
   * 全て選択 選択時
   */
  public void cmdSel_all() {
    this.table.selectAll();
  }

  /**
   * 選択解除 選択時
   */
  public void cmdSel_clear() {
    this.table.deselectAll();
  }

  // /**
  // * テーブル配置時の列スパン数指定
  // * @param span
  // */
  // public void setHorizontalSpan(int span) {
  // // GridData grid = (GridData) this.table.getLayoutData();
  // GridData grid = (GridData) getLayoutData();
  // grid.horizontalSpan = span;
  // setLayoutData(grid);
  // }
}
