package jp.co.powerbeans.vbscore.swt.parts;

import jp.co.powerbeans.common.swt.CmdSelectionAdapter;
import jp.co.powerbeans.common.swt.PBComposite;
import jp.co.powerbeans.common.swt.PBControlUtil;
import jp.co.powerbeans.vbscore.CMD;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.db.DBCacheManager;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.logic.ScoreMgr;
import jp.co.powerbeans.vbscore.model.ActionDef;
import jp.co.powerbeans.vbscore.model.ActionResultDef;
import jp.co.powerbeans.vbscore.model.Game;
import jp.co.powerbeans.vbscore.model.GameSet;
import jp.co.powerbeans.vbscore.model.Play;
import jp.co.powerbeans.vbscore.model.Player;
import jp.co.powerbeans.vbscore.model.Team;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

/**
 * プレー入力パネルComposit.
 * プレーから先に入力する
 * @author amonden
 *
 */
public class InputPlayComposit extends PBComposite {

//	private Table playerTable;
	/**
	 * 自チーム
	 */
	private Team myTeam;
	/**
	 * プレー中の自チーム選手
	 */
	private Player[] playingPlayers;
	/**
	 * 現在のSet
	 */
	private GameSet set;
	/**
	 * プレーアクションリスト
	 */
	private ActionDef[] actionDefs;
	/**
	 * プレー選択ボタンイベントリスナー
	 */
	private InputPlayCompositEventListener playListListener;
	/**
	 * 現在のGame
	 */
	private Game game;

	/**
	 * コンストラクタ
	 * @param shell 親Shell
	 * @param game Game
	 * @param set Set
	 * @param myTeam 自チーム
	 * @param playListListener プレー選択ボタンイベントリスナー
	 */
	public InputPlayComposit(Shell shell, Game game, GameSet set, Team myTeam, InputPlayCompositEventListener playListListener) {
		super(shell, SWT.NO_FOCUS);
		this.myTeam = myTeam;
		this.game = game;
		this.set = set;
		this.playListListener = playListListener;
		loadData();
		
		setLayout(new GridLayout(1, true));
		GridData compGrid = new GridData();
		compGrid.horizontalAlignment = GridData.FILL;
		compGrid.verticalAlignment = GridData.FILL;
		compGrid.grabExcessHorizontalSpace = true;
		compGrid.grabExcessVerticalSpace = true;
		compGrid.heightHint = 200;
		setLayoutData(compGrid);
		setBackground(new Color(getDisplay(), 255, 255, 255));

		PBControlUtil.createLabel_left(this, Msg.getInputPlayPanel());
		for (int i = 0; i < this.actionDefs.length; i++) {
			ActionDef a = this.actionDefs[i];
			PBControlUtil.createSimpleButton(this, "&" + a.getActionDefId() + ":" + a.getActionName(), CMD.SEL_ACTION, new Object[]{a});
		}
	}

	/**
	 * 選手データ、アクションデータロード
	 */
	private void loadData() {
		// プレー中の選手を取得
		updatePlayingPlayer();
		
		// アクション一覧を取得
		this.actionDefs = (ActionDef[]) DBManager.findByAll(ActionDef.class);
		
	}
	
	/**
	 * ボタン選択時
	 * @param actiondef アクション
	 */
	public void cmdSel_action(ActionDef actiondef) {
		showPlayInputMenu(actiondef);
	}

	private void showPlayInputMenu(ActionDef actiondef) {
		Menu menu = new Menu(this);
		
		// actionDefId と一致するActionResultの名称を格納
		ActionResultDef[] results = (ActionResultDef[]) DBCacheManager.findBy(ActionResultDef.class, "actionDefId = " + actiondef.getActionDefId(), 
					"actionResultDefId");
		
		for (int i = 0; i < results.length; i++) {
			MenuItem menuItem0 = new MenuItem(menu, SWT.CASCADE);
			menuItem0.setText("&" + (i + 1) + results[i].getActionResultName());
			menuItem0.setData(results[i]);
			createPlayerMenu(menuItem0, actiondef, results[i]);
		}
	
		this.setMenu(menu);
		menu.setVisible(true);
	}

	private Menu createPlayerMenu(MenuItem parentmenu, ActionDef actiondef, ActionResultDef res) {
		Menu menu = new Menu(parentmenu);
		parentmenu.setMenu(menu);
		for (int i = 0; i < this.playingPlayers.length; i++) {
			Player p = this.playingPlayers[i];
			if (p == null) {
				PBControlUtil.addSeparator(menu);
			} else {
				MenuItem menuItem0 = new MenuItem(menu, SWT.PUSH);
				menuItem0.setText("&" + (i + 1) + " " + p.getStdNumber() + " " + p.getPlayerName());
	 			menuItem0.addSelectionListener(new CmdSelectionAdapter(this, CMD.SEL_PLAYER,
	 					new Object[]{actiondef, res, p}));
			}
		}
		
		return menu;
	}
	
	/**
	 * 自チームのアクション結果を選択
	 * @param def
	 * @param p
	 */
	public void cmdSel_player(ActionDef actiondef, ActionResultDef def, Player player) {
		
		// 0. 追加する Play を生成
		Play p = new Play();
		p.setActionDefId(def.getActionDefId());
		p.setActionPlayerId(player.getPlayerId());
		p.setActionPlayerTeamId(player.getTeamId());
		p.setActionResultDefId(def.getActionResultDefId());
		p.setGameSetId(this.set.getGameSetId());
		if (
		    (def.getPoint() == 1 && p.getActionPlayerTeamId() == this.myTeam.getTeamId()) ||
		    (def.getPoint() == -1 && p.getActionPlayerTeamId() != this.myTeam.getTeamId())) {
			p.setMyTeamAddPoint(1);
		} else if (def.getPoint() != 0) {
			p.setVsTeamAddPoint(1);
		}
		p.setMyTeamSumPoint(this.set.getMyResultScore() + p.getMyTeamAddPoint());
		p.setVsTeamSumPoint(this.set.getVsResultScore() + p.getVsTeamAddPoint());
		this.set.setMyResultScore(p.getMyTeamSumPoint());
		this.set.setVsResultScore(p.getVsTeamSumPoint());
		
		// 1. 表示用リストのアクションリストに追加
		this.playListListener.addPlay(p);
		
		// 2. 保存
		DBManager.insert(p);
		DBManager.update(this.set);
	}

	/**
	 * プレー中プレイヤーリスト取得
	 * @return playingPlayers を戻します。
	 */
	public Player[] getPlayingPlayers() {
		return playingPlayers;
	}

	/**
	 * プレイングプレイヤーをDBから更新
	 */
	public void updatePlayingPlayer() {
		// プレー中の選手を取得
		this.playingPlayers = ScoreMgr.copySetToMyPlayingPlayers(this.set, true, this.game.getVsTeamId());
	}
}
